@@ -37,7 +37,8 @@ WARC::Record - one record from a WARC file
 
   $fields = $record->fields;
 
-...
+  # Supply a data block for an in-memory record
+  $warcinfo->block(new WARC::Fields ( ... ));
 
 =cut
 
@@ -49,6 +50,11 @@ use overload fallback => 1;
 #
 #   fields
 #	Embedded WARC::Fields object
+#
+#  Keys defined by this class but overridden and unused in subclasses:
+#
+#   block
+#	Data block for this record
 
 sub _dbg_dump {
   my $self = shift;
@@ -60,15 +66,6 @@ sub _dbg_dump {
   return $out;
 }
 
-# This method can be overridden in subclasses for read-only objects.
-sub _set {
-  my $self = shift;
-  my $key = shift;
-  my $value = shift;
-
-  $self->{$key} = $value;
-}
-
 =head1 DESCRIPTION
 
 C<WARC::Record> objects come in two flavors with a common interface.
@@ -154,6 +151,14 @@ Alias for C<$record-E<gt>field('WARC-Record-ID')>.
 
 sub id { (shift)->field('WARC-Record-ID') }
 
+=item $record-E<gt>content_length
+
+Alias for C<$record-E<gt>field('Content-Length')>.
+
+=cut
+
+sub content_length { (shift)->field('Content-Length') }
+
 =item $record-E<gt>date
 
 Return the C<'WARC-Date'> field as a C<WARC::Date> object.
@@ -283,8 +288,8 @@ Return a tied filehandle that reads the WARC record payload.
 
 The WARC record payload is defined as the decoded content of the protocol
 response or other resource stored in the record.  This method returns undef
-if called on a WARC record that has no payload or content that we do not
-recognize.
+if called on a WARC record that has no payload or that has content that we
+do not recognize.
 
 =cut
 
@@ -314,6 +319,41 @@ sub new {
   bless { fields => $fields }, $class;
 }
 
+=item $record-E<gt>block
+
+=item $record-E<gt>block( $new_value )
+
+Get or set the block contents of an in-memory record.  This method returns
+undef if called on a WARC record from a volume and croaks if setting the
+contents is attempted on a record from a volume.
+
+=cut
+
+sub block {
+  my $self = shift;
+  my $block = $self->{block};
+
+  if (@_) {
+    my $new = shift;
+
+    if (ref $new) {
+      if ($new->can('as_block'))
+	{ $self->{block} = $new->as_block }
+      elsif ($new->isa('HTTP::Message'))
+	{ $self->{block} = $new->as_string(WARC::CRLF()) }
+      elsif ($new->can('as_string'))
+	{ $self->{block} = $new->as_string }
+      else
+	{ croak "unrecognized object submitted as WARC record block" }
+    } else { $self->{block} = $new }
+
+    use bytes;
+    $self->{fields}->field('Content-Length', length $self->{block});
+  }
+
+  return $block;
+}
+
 =back
 
 =cut
